using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.OpenGL.Programs
{
    public abstract class GLProgram : IDisposable
    {
        private WindowContext context;
        internal readonly UIntPtr handle;
        private protected GLCreateProgram createProgram;
        private protected GLDeleteProgram deleteProgram;
        private protected GLLinkProgram linkProgram;
        private protected GLGetProgramInfoLog getProgramInfoLog;
        private protected GLProgramParameter programParameter;
        private protected GLUseProgram useProgram;

        public GLProgram(bool separable = true) {
            this.context = WindowContextsManager.CurrentWindowContext();
            createProgram = Marshal.GetDelegateForFunctionPointer<GLCreateProgram>(SDL.SDL_GL_GetProcAddress("glCreateProgram"));
            if (createProgram == null) throw new FrameworkSDLException();
            deleteProgram = Marshal.GetDelegateForFunctionPointer<GLDeleteProgram>(SDL.SDL_GL_GetProcAddress("glDeleteProgram"));
            if (deleteProgram == null) throw new FrameworkSDLException();
            linkProgram = Marshal.GetDelegateForFunctionPointer<GLLinkProgram>(SDL.SDL_GL_GetProcAddress("glLinkProgram"));
            if (linkProgram == null) throw new FrameworkSDLException();
            getProgramInfoLog = Marshal.GetDelegateForFunctionPointer<GLGetProgramInfoLog>(SDL.SDL_GL_GetProcAddress("glGetProgramInfoLog"));
            if (getProgramInfoLog == null) throw new FrameworkSDLException();
            programParameter = Marshal.GetDelegateForFunctionPointer<GLProgramParameter>(SDL.SDL_GL_GetProcAddress("glProgramParameter"));
            if (programParameter == null) throw new FrameworkSDLException();
            useProgram = Marshal.GetDelegateForFunctionPointer<GLUseProgram>(SDL.SDL_GL_GetProcAddress("glUseProgram"));
            if (useProgram == null) throw new FrameworkSDLException();

            handle = createProgram();
            if (separable) {
                programParameter(handle, GLEnums.GL_PROGRAM_SEPARABLE, (int) GLEnums.GL_TRUE);
            }
        }

        public virtual void Use() {
            useProgram(handle);
        }

        public virtual void Dispose()
        {
            deleteProgram(handle);
        }

        ~GLProgram() {
            Dispose();
        }
    }
}