using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.OpenGL.Shaders;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.OpenGL.Programs
{
    public abstract class GLProgramPipeline : IDisposable {
        private readonly WindowContext associatedContext;
        private UIntPtr[] pipelines;
        private protected UIntPtr PipelineHandle { get {return pipelines[0]; } }
        private protected GLGenProgramPipelines genProgramPipelines;
        private protected GLDeleteProgramPipelines deleteProgramPipelines;
        private protected GLBindProgramPipeline bindProgramPipeline;
        private protected GLUseProgramStages useProgramStages;
        private protected GLUseProgram useProgram;
        public GLProgramPipeline() {
            associatedContext = WindowContextsManager.CurrentWindowContext();
            genProgramPipelines = Marshal.GetDelegateForFunctionPointer<GLGenProgramPipelines>(SDL.SDL_GL_GetProcAddress("glGenProgramPipelines"));
            if (genProgramPipelines == null) throw new FrameworkSDLException();
            useProgramStages = Marshal.GetDelegateForFunctionPointer<GLUseProgramStages>(SDL.SDL_GL_GetProcAddress("glUseProgramStages"));
            if (useProgramStages == null) throw new FrameworkSDLException();
            useProgram = Marshal.GetDelegateForFunctionPointer<GLUseProgram>(SDL.SDL_GL_GetProcAddress("glUseProgram"));
            if (useProgram == null) throw new FrameworkSDLException();
            bindProgramPipeline = Marshal.GetDelegateForFunctionPointer<GLBindProgramPipeline>(SDL.SDL_GL_GetProcAddress("glBindProgramPipeline"));
            if (bindProgramPipeline == null) throw new FrameworkSDLException();
            deleteProgramPipelines = Marshal.GetDelegateForFunctionPointer<GLDeleteProgramPipelines>(SDL.SDL_GL_GetProcAddress("glDeleteProgramPipelines"));
            if (deleteProgramPipelines == null) throw new FrameworkSDLException();

            genProgramPipelines(1, out pipelines);

        }

        public virtual void Use() {
            useProgram(UIntPtr.Zero);
        }

        public virtual void Dispose()
        {
            deleteProgramPipelines((uint) pipelines.Length, pipelines);
        }

        ~GLProgramPipeline() {
            Dispose();
        }
    }
}