using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.OpenGL.Shaders;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.OpenGL.Programs
{
    public class GLShaderProgram : GLProgram
    {
        private readonly GLShader[] shaders;
        private readonly GLAttachShader attachShader;
        private readonly GLDetachShader detachShader;
        private GLBindAttribLocation bindAttribLocation;
        public GLShaderProgram(bool separable = true, params GLShader[] shaders) : base(separable) {
            if (shaders.Length == 0) throw new ArgumentException("Requires at least one shader for shader program.");
            this.shaders = shaders;
            attachShader = Marshal.GetDelegateForFunctionPointer<GLAttachShader>(SDL.SDL_GL_GetProcAddress("glAttachShader"));
            if (attachShader == null) throw new FrameworkSDLException();
            bindAttribLocation = Marshal.GetDelegateForFunctionPointer<GLBindAttribLocation>(SDL.SDL_GL_GetProcAddress("glBindAttribLocation"));
            if (bindAttribLocation == null) throw new FrameworkSDLException();
            detachShader = Marshal.GetDelegateForFunctionPointer<GLDetachShader>(SDL.SDL_GL_GetProcAddress("glDetachShader"));
            if (detachShader == null) throw new FrameworkSDLException();

            foreach (GLShader shader in shaders)
            {
                attachShader(handle, shader.Handle);
            }

            linkProgram(handle);
            uint length;
            string log;
            getProgramInfoLog(handle, 1024, out length, out log);
            if (length > 0) {
                Dispose();
                throw new OpenGLException(log);
            }

            foreach (GLShader shader in shaders)
            {
                detachShader(handle, shader.Handle);
            }
        }

        public override void Dispose() {
            foreach (GLShader shader in shaders)
            {
                shader.Dispose();
            }
            base.Dispose();
        }

        ~GLShaderProgram() {
            Dispose();
        }
    }
}