using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.OpenGL.Shaders
{
    public abstract class GLShader : IDisposable {
        private WindowContext context;
        public UIntPtr Handle { get; private protected set; }

        private protected GLCreateShader createShader;
        private protected GLShaderSource shaderSource;
        private protected GLCompileShader compileShader;
        private protected GLGetShaderLogInfo getShaderLogInfo;
        private protected GLDeleteShader deleteShader;

        public GLShader() {
            context = WindowContextsManager.CurrentWindowContext();
            createShader = Marshal.GetDelegateForFunctionPointer<GLCreateShader>(SDL.SDL_GL_GetProcAddress("glCreateShader"));
            if (createShader == null) throw new FrameworkSDLException();
            shaderSource = Marshal.GetDelegateForFunctionPointer<GLShaderSource>(SDL.SDL_GL_GetProcAddress("glShaderSource"));
            if (shaderSource == null) throw new FrameworkSDLException();
            compileShader = Marshal.GetDelegateForFunctionPointer<GLCompileShader>(SDL.SDL_GL_GetProcAddress("glCompileShader"));
            if (compileShader == null) throw new FrameworkSDLException();
            getShaderLogInfo = Marshal.GetDelegateForFunctionPointer<GLGetShaderLogInfo>(SDL.SDL_GL_GetProcAddress("glGetShaderLogInfo"));
            if(getShaderLogInfo == null) throw new FrameworkSDLException();
            deleteShader = Marshal.GetDelegateForFunctionPointer<GLDeleteShader>(SDL.SDL_GL_GetProcAddress("glDeleteShader"));
            if (deleteShader == null) throw new FrameworkSDLException();

        }
        
        public virtual void Dispose()
        {
        }

        ~GLShader() {
            Dispose();
        }
    }
}